'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

// todo: this class has been taken from bit-bin. The class should be exported as a bit component once 'virtualization' changes are done.

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _exceptions = require('./exceptions');

var _constants = require('../constants');

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var BitId = function () {
  function BitId(_ref) {
    var scope = _ref.scope,
        box = _ref.box,
        name = _ref.name,
        version = _ref.version;

    _classCallCheck(this, BitId);

    this.scope = scope || null;
    this.box = box || 'global';
    this.name = name;
    this.version = version || null;
  }

  _createClass(BitId, [{
    key: 'changeScope',
    value: function changeScope(newScope) {
      return new BitId({ scope: newScope, box: this.box, name: this.name, version: this.version });
    }
  }, {
    key: 'getScopeWithoutRemoteAnnotation',
    value: function getScopeWithoutRemoteAnnotation() {
      return this.scope ? this.scope.replace(_constants.REMOTE_ALIAS_SIGN, '') : this.scope;
    }
  }, {
    key: 'isLocal',
    value: function isLocal(scopeName) {
      return this.scope === null || scopeName === this.getScopeWithoutRemoteAnnotation();
    }
  }, {
    key: 'toString',
    value: function toString() {
      var ignoreScope = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var name = this.name,
          box = this.box,
          version = this.version;

      var scope = this.scope;
      var componentStr = ignoreScope || !scope ? [box, name].join('/') : [scope, box, name].join('/');
      if (version && scope) {
        return componentStr.concat(`${_constants.VERSION_DELIMITER}${version}`);
      }

      return componentStr;
    }
  }, {
    key: 'toObject',
    value: function toObject() {
      var key = [this.scope, this.box, this.name].join('/');
      var value = this.version;

      return { [key]: value };
    }
  }, {
    key: 'toPath',
    value: function toPath() {
      // todo: change according to the resolve-conflict strategy
      return _path2.default.join(this.box, this.name, this.scope, this.version);
    }
  }, {
    key: 'toFullPath',
    value: function toFullPath() {
      return _path2.default.join(this.box, this.name, this.scope, this.version);
    }
  }], [{
    key: 'parse',
    value: function parse(id, realScopeName) {
      var version = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _constants.LATEST_VERSION;

      if (!id || id === _constants.NO_PLUGIN_TYPE) {
        return null;
      }
      if (id.includes(_constants.VERSION_DELIMITER)) {
        var _id$split = id.split(_constants.VERSION_DELIMITER),
            _id$split2 = _slicedToArray(_id$split, 2),
            newId = _id$split2[0],
            newVersion = _id$split2[1];

        id = newId;
        version = newVersion;
      }

      var splited = id.split('/');
      if (splited.length === 3) {
        var _splited = _slicedToArray(splited, 3),
            _scope = _splited[0],
            _box = _splited[1],
            _name = _splited[2];

        if (_scope === _constants.LOCAL_SCOPE_NOTATION && !realScopeName) {
          throw new Error('real scope name is required in bitId.parse with @this notation');
        }
        var digestScopeName = _scope === _constants.LOCAL_SCOPE_NOTATION ? realScopeName : _scope;
        if (!(0, _utils.isValidIdChunk)(_name) || !(0, _utils.isValidIdChunk)(_box) || !(0, _utils.isValidScopeName)(digestScopeName)) {
          // $FlowFixMe
          throw new _exceptions.InvalidIdChunk(`${digestScopeName}/${_box}/${_name}`);
        }
        // $FlowFixMe (in this case the realScopeName is not null)
        return new BitId({
          scope: digestScopeName,
          box: _box,
          name: _name,
          version
        });
      }

      if (splited.length === 2) {
        var _splited2 = _slicedToArray(splited, 2),
            _box2 = _splited2[0],
            _name2 = _splited2[1];

        if (!(0, _utils.isValidIdChunk)(_name2) || !(0, _utils.isValidScopeName)(_box2)) {
          // $FlowFixMe
          throw new _exceptions.InvalidIdChunk(`${_box2}/${_name2}`);
        }
        // $FlowFixMe
        return new BitId({
          box: _box2,
          name: _name2,
          version
        });
      }

      if (splited.length === 1) {
        var _splited3 = _slicedToArray(splited, 1),
            _name3 = _splited3[0];

        if (!(0, _utils.isValidIdChunk)(_name3)) {
          // $FlowFixMe
          throw new _exceptions.InvalidIdChunk(_name3);
        }
        // $FlowFixMe
        return new BitId({
          name: _name3,
          version
        });
      }

      throw new _exceptions.InvalidBitId();
    }
  }]);

  return BitId;
}();

exports.default = BitId;
//# sourceMappingURL=bit-id.js.map