'use strict';

/**
* this file had been forked from https://github.com/pahen/detective-typescript
*/

var Parser = require('typescript-eslint-parser');
var Walker = require('node-source-walk');

/**
 * Extracts the dependencies of the supplied TypeScript module
 *
 * @param  {String|Object} src - File's content or AST
 * @param  {Object} options - options to pass to the parser
 * @return {String[]}
 */
module.exports = function (src) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  options.parser = Parser;

  var walker = new Walker(options);

  var dependencies = {};
  var addDependency = function addDependency(dependency) {
    if (!dependencies[dependency]) {
      dependencies[dependency] = {};
    }
  };
  var addImportSpecifier = function addImportSpecifier(dependency, importSpecifier) {
    if (dependencies[dependency].importSpecifiers) {
      dependencies[dependency].importSpecifiers.push(importSpecifier);
    } else {
      dependencies[dependency].importSpecifiers = [importSpecifier];
    }
  };

  if (typeof src === 'undefined') {
    throw new Error('src not given');
  }

  if (src === '') {
    return dependencies;
  }

  walker.walk(src, function (node) {
    switch (node.type) {
      case 'ImportDeclaration':
        if (node.source && node.source.value) {
          var dependency = node.source.value;
          addDependency(dependency);

          node.specifiers.forEach(function (specifier) {
            var specifierValue = {
              isDefault: specifier.type === 'ImportDefaultSpecifier',
              name: specifier.local.name
            };
            addImportSpecifier(dependency, specifierValue);
          });
        }
        break;
      case 'ExportNamedDeclaration':
      case 'ExportAllDeclaration':
        if (node.source && node.source.value) {
          addDependency(node.source.value);
        }
        break;
      case 'TSExternalModuleReference':
        if (node.expression && node.expression.value) {
          addDependency(node.expression.value);
        }
        break;
      default:
        break;
    }
  });

  return dependencies;
};
//# sourceMappingURL=index.js.map