'use strict';

var fs = require('fs');
var path = require('path');
var compiler = require('vue-template-compiler');

var DEFAULT_SCRIPT_LANG = 'js';
var DEFAULT_STYLE_LANG = 'scss';

var languageMap = {
  css: "scss",
  stylus: "styl"
};
module.exports = function (partial, filename, directory, config, webpackConfig, configPath, ast, isScript, content, resolveConfig) {
  var cabinet = require('../../filing-cabinet');

  var fileContent = fs.readFileSync(filename);

  var _compiler$parseCompon = compiler.parseComponent(fileContent.toString(), { pad: 'line' }),
      script = _compiler$parseCompon.script,
      styles = _compiler$parseCompon.styles;

  if (isScript) {
    var scriptExt = script.lang ? languageMap[script.lang] || script.lang : DEFAULT_SCRIPT_LANG;
    return cabinet({
      partial: partial,
      filename: filename,
      directory: path.dirname(filename),
      content: script.content,
      resolveConfig,
      ext: `.${scriptExt}` || path.extname(partial)
    });
  }
  var stylesResult = styles.map(function (style) {
    var styleExt = style.lang ? languageMap[style.lang] || style.lang : DEFAULT_STYLE_LANG;
    return cabinet({
      partial: partial,
      filename: `${path.join(path.dirname(filename), path.parse(filename).name)}.${styleExt}`,
      directory: path.dirname(filename),
      content: style.content,
      resolveConfig,
      ext: `.${styleExt}`
    });
  });
  return stylesResult[0];
};
//# sourceMappingURL=index.js.map