'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _parents = require('parents');

var _parents2 = _interopRequireDefault(_parents);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _exceptions = require('../exceptions');

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function composePath(componentRootFolder) {
  return _path2.default.join(componentRootFolder, _constants.PACKAGE_JSON);
}
function convertComponentsIdToValidPackageName(registryPrefix, id) {
  return `${registryPrefix}/${id.replace(/\//g, '.')}`;
}
function convertComponentsToValidPackageNames(registryPrefix, bitDependencies) {
  var obj = {};
  if (_ramda2.default.isEmpty(bitDependencies) || _ramda2.default.isNil(bitDependencies)) return obj;
  Object.keys(bitDependencies).forEach(function (key) {
    var name = convertComponentsIdToValidPackageName(registryPrefix, key);
    obj[name] = bitDependencies[key];
  });
  return obj;
}

var PackageJsonPropsNames = ['name', 'version', 'homepage', 'main', 'dependencies', 'devDependencies', 'peerDependencies', 'license', 'scripts', 'workspaces', 'private'];

var PackageJson = function () {
  function PackageJson(componentRootFolder, _ref) {
    var name = _ref.name,
        version = _ref.version,
        homepage = _ref.homepage,
        main = _ref.main,
        dependencies = _ref.dependencies,
        devDependencies = _ref.devDependencies,
        peerDependencies = _ref.peerDependencies,
        license = _ref.license,
        scripts = _ref.scripts,
        workspaces = _ref.workspaces;

    _classCallCheck(this, PackageJson);

    this.name = name;
    this.version = version;
    this.homepage = homepage;
    this.main = main;
    this.dependencies = dependencies;
    this.devDependencies = devDependencies;
    this.peerDependencies = peerDependencies;
    this.componentRootFolder = componentRootFolder;
    this.license = license;
    this.scripts = scripts;
    this.workspaces = workspaces;
  } // path where to write the package.json


  _createClass(PackageJson, [{
    key: 'toPlainObject',
    value: function toPlainObject() {
      var self = this;
      var result = {};
      var addToResult = function addToResult(propName) {
        result[propName] = self[propName];
      };

      _ramda2.default.forEach(addToResult, PackageJsonPropsNames);
      if (this.workspaces) result.private = true;
      return result;
    }
  }, {
    key: 'toJson',
    value: function toJson() {
      var readable = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

      if (!readable) return JSON.stringify(this.toPlainObject());
      return JSON.stringify(this.toPlainObject(), null, 4);
    }
  }, {
    key: 'addDependencies',
    value: function addDependencies(bitDependencies, registryPrefix) {
      this.dependencies = Object.assign({}, this.dependencies, convertComponentsToValidPackageNames(registryPrefix, bitDependencies));
    }
  }, {
    key: 'addDevDependencies',
    value: function addDevDependencies(bitDevDependencies, registryPrefix) {
      this.devDependencies = Object.assign({}, this.devDependencies, convertComponentsToValidPackageNames(registryPrefix, bitDevDependencies));
    }
  }, {
    key: 'write',
    value: function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee(_ref2) {
        var _ref2$override = _ref2.override,
            override = _ref2$override === undefined ? true : _ref2$override;
        var plain;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!(!override && PackageJson.hasExisting(this.componentRootFolder))) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', Promise.reject(new _exceptions.PackageJsonAlreadyExists(this.componentRootFolder)));

              case 2:
                plain = this.toPlainObject();
                return _context.abrupt('return', _fsExtra2.default.outputJSON(composePath(this.componentRootFolder), plain, { spaces: 2 }));

              case 4:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function write(_x2) {
        return _ref3.apply(this, arguments);
      }

      return write;
    }()
  }], [{
    key: 'hasExisting',
    value: function hasExisting(componentRootFolder) {
      var throws = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      var packageJsonPath = composePath(componentRootFolder);
      var exists = _fsExtra2.default.pathExistsSync(packageJsonPath);
      if (!exists && throws) {
        throw new _exceptions.PackageJsonNotFound(packageJsonPath);
      }
      return exists;
    }
  }, {
    key: 'create',
    value: function create(componentRootFolder) {
      return new PackageJson(componentRootFolder, {});
    }
  }, {
    key: 'ensure',
    value: function ensure(componentRootFolder) {
      return this.load(componentRootFolder);
    }
  }, {
    key: 'fromPlainObject',
    value: function fromPlainObject(componentRootFolder, object) {
      return new PackageJson(componentRootFolder, object);
    }
  }, {
    key: 'load',
    value: function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee2(componentRootFolder) {
        var throwError = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
        var composedPath, componentJsonObject;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                composedPath = composePath(componentRootFolder);

                if (PackageJson.hasExisting(componentRootFolder, throwError)) {
                  _context2.next = 3;
                  break;
                }

                return _context2.abrupt('return', null);

              case 3:
                _context2.next = 5;
                return _fsExtra2.default.readJson(composedPath);

              case 5:
                componentJsonObject = _context2.sent;
                return _context2.abrupt('return', new PackageJson(componentRootFolder, componentJsonObject));

              case 7:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function load(_x5) {
        return _ref4.apply(this, arguments);
      }

      return load;
    }()
  }, {
    key: 'loadSync',
    value: function loadSync(componentRootFolder) {
      var throwError = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

      var composedPath = composePath(componentRootFolder);
      if (!PackageJson.hasExisting(componentRootFolder, throwError)) return null;
      var componentJsonObject = _fsExtra2.default.readJsonSync(composedPath);
      return new PackageJson(componentRootFolder, componentJsonObject);
    }

    /**
     * Taken from this package (with some minor changes):
     * https://www.npmjs.com/package/find-package
     * https://github.com/jalba/find-package
     */

  }, {
    key: 'findPath',
    value: function findPath(dir) {
      var parentsArr = (0, _parents2.default)(dir);
      var i = void 0;
      for (i = 0; i < parentsArr.length; i++) {
        var config = `${parentsArr[i]}/package.json`;
        try {
          if (_fsExtra2.default.lstatSync(config).isFile()) {
            return config;
          }
        } catch (e) {}
      }
      return null;
    }

    /**
     * Taken from this package (with some minor changes):
     * https://www.npmjs.com/package/find-package
     * https://github.com/jalba/find-package
     *
     */

  }, {
    key: 'findPackage',
    value: function findPackage(dir, addPaths) {
      var pathToConfig = this.findPath(dir);
      var configJSON = null;
      if (pathToConfig !== null) configJSON = require(pathToConfig);
      if (configJSON && addPaths) {
        configJSON.paths = {
          relative: _path2.default.relative(dir, pathToConfig),
          absolute: pathToConfig
        };
      } else if (configJSON !== null) {
        delete configJSON.paths;
      }

      return configJSON;
    }

    /*
    * load package.json from path
    */

  }, {
    key: 'getPackageJson',
    value: function getPackageJson(path) {
      var getRawObject = function getRawObject() {
        return _fsExtra2.default.readJson(composePath(path));
      };
      var exist = PackageJson.hasExisting(path);
      if (exist) return getRawObject();
    }

    /*
     * save package.json in path
     */

  }, {
    key: 'saveRawObject',
    value: function saveRawObject(path, obj) {
      return _fsExtra2.default.outputJSON(composePath(path), obj, { spaces: 2 });
    }

    /*
     * For an existing package.json file of the root project, we don't want to do any change, other than what needed.
     * That's why this method doesn't use the 'load' and 'write' methods of this class. Otherwise, it'd write only the
     * PackageJsonPropsNames attributes.
     * Also, in case there is no package.json file in this project, it generates a new one with only the 'dependencies'
     * attribute. Nothing more, nothing less.
     */

  }, {
    key: 'addComponentsIntoExistingPackageJson',
    value: function () {
      var _ref5 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee3(rootDir, components, registryPrefix) {
        var packageJson;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return PackageJson.getPackageJson(rootDir);

              case 2:
                _context3.t0 = _context3.sent;

                if (_context3.t0) {
                  _context3.next = 5;
                  break;
                }

                _context3.t0 = { dependencies: {} };

              case 5:
                packageJson = _context3.t0;

                packageJson.dependencies = Object.assign({}, packageJson.dependencies, convertComponentsToValidPackageNames(registryPrefix, components));
                _context3.next = 9;
                return PackageJson.saveRawObject(rootDir, packageJson);

              case 9:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function addComponentsIntoExistingPackageJson(_x7, _x8, _x9) {
        return _ref5.apply(this, arguments);
      }

      return addComponentsIntoExistingPackageJson;
    }()
    /*
     * For an existing package.json file of the root project, we don't want to do any change, other than what needed.
     * That's why this method doesn't use the 'load' and 'write' methods of this class. Otherwise, it'd write only the
     * PackageJsonPropsNames attributes.
     * Also, in case there is no package.json file in this project, it generates a new one with only the 'dependencies'
     * adds workspaces with private flag if dosent exist.
     */

  }, {
    key: 'addWorkspacesToPackageJson',
    value: function () {
      var _ref6 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee4(rootDir, componentsDefaultDirectory, dependenciesDirectory, customImportPath) {
        var pkg, workSpaces;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return PackageJson.getPackageJson(rootDir);

              case 2:
                _context4.t0 = _context4.sent;

                if (_context4.t0) {
                  _context4.next = 5;
                  break;
                }

                _context4.t0 = {};

              case 5:
                pkg = _context4.t0;
                workSpaces = pkg.workspaces || [];

                workSpaces.push(dependenciesDirectory);
                workSpaces.push(componentsDefaultDirectory);
                if (customImportPath) workSpaces.push(customImportPath);
                pkg.workspaces = _ramda2.default.uniq(workSpaces);
                pkg.private = !!pkg.workspaces;
                _context4.next = 14;
                return PackageJson.saveRawObject(rootDir, pkg);

              case 14:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function addWorkspacesToPackageJson(_x10, _x11, _x12, _x13) {
        return _ref6.apply(this, arguments);
      }

      return addWorkspacesToPackageJson;
    }()

    /*
     * remove workspaces dir from workspace in package.json with changing other fields in package.json
     */

  }, {
    key: 'removeComponentsFromWorkspaces',
    value: function () {
      var _ref7 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee5(rootDir, pathsTOoRemove) {
        var pkg, workSpaces;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return PackageJson.getPackageJson(rootDir);

              case 2:
                _context5.t0 = _context5.sent;

                if (_context5.t0) {
                  _context5.next = 5;
                  break;
                }

                _context5.t0 = {};

              case 5:
                pkg = _context5.t0;
                workSpaces = pkg.workspaces || [];

                pkg.workspaces = workSpaces.filter(function (folder) {
                  return !pathsTOoRemove.includes(folder);
                });
                _context5.next = 10;
                return PackageJson.saveRawObject(rootDir, pkg);

              case 10:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function removeComponentsFromWorkspaces(_x14, _x15) {
        return _ref7.apply(this, arguments);
      }

      return removeComponentsFromWorkspaces;
    }()

    /*
     * remove components from package.json dependencies
     */

  }, {
    key: 'removeComponentsFromDependencies',
    value: function () {
      var _ref8 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee6(rootDir, registryPrefix, componentIds) {
        var pkg;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.next = 2;
                return PackageJson.getPackageJson(rootDir);

              case 2:
                pkg = _context6.sent;

                if (!(pkg && pkg.dependencies)) {
                  _context6.next = 7;
                  break;
                }

                componentIds.forEach(function (id) {
                  delete pkg.dependencies[convertComponentsIdToValidPackageName(registryPrefix, id)];
                });
                _context6.next = 7;
                return PackageJson.saveRawObject(rootDir, pkg);

              case 7:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function removeComponentsFromDependencies(_x16, _x17, _x18) {
        return _ref8.apply(this, arguments);
      }

      return removeComponentsFromDependencies;
    }()
  }]);

  return PackageJson;
}();

exports.default = PackageJson;
//# sourceMappingURL=package-json.js.map