'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = npmLogin;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _iniBuilder = require('ini-builder');

var _iniBuilder2 = _interopRequireDefault(_iniBuilder);

var _userHome = require('user-home');

var _userHome2 = _interopRequireDefault(_userHome);

var _constants = require('../constants');

var _exceptions = require('./exceptions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function findrc(pathToNpmrc) {
  var userNpmrc = _path2.default.join(_userHome2.default, '.npmrc');
  if (pathToNpmrc) {
    if (!_fs2.default.existsSync(pathToNpmrc)) throw new _exceptions.PathToNpmrcNotExist(pathToNpmrc);
    var stats = _fs2.default.statSync(pathToNpmrc);
    if (stats.isFile()) userNpmrc = pathToNpmrc;else userNpmrc = _path2.default.join(pathToNpmrc, '.npmrc');
  }
  return userNpmrc;
}

function mergeOrCreateConfig(token, url) {
  var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  var strippedUrl = url.replace(/(^\w+:|^)\/\//, '');
  var iniReg = _iniBuilder2.default.find(config, `${_constants.DEFAULT_BINDINGS_PREFIX}:registry`);
  var iniToken = _iniBuilder2.default.find(config, `//${strippedUrl}/:_authToken`);
  if (!iniReg) {
    config.push({
      path: [`${_constants.DEFAULT_BINDINGS_PREFIX}:registry`],
      value: url
    });
  } else {
    iniReg.value = url;
  }
  if (!iniToken) {
    config.push({
      path: [`//${strippedUrl}/:_authToken`],
      value: token
    });
  } else {
    iniToken.value = token;
  }
  return config;
}

function npmLogin(token, pathToNpmrc, url) {
  var npmrcPath = findrc(pathToNpmrc);
  var npmrcConfig = _fs2.default.existsSync(npmrcPath) ? mergeOrCreateConfig(token, url, _iniBuilder2.default.parse(_fs2.default.readFileSync(npmrcPath, 'utf-8'))) : mergeOrCreateConfig(token, url);
  try {
    _fs2.default.writeFileSync(npmrcPath, _iniBuilder2.default.serialize(npmrcConfig));
  } catch (err) {
    throw new _exceptions.WriteToNpmrcError(npmrcPath);
  }
  return npmrcPath;
}
//# sourceMappingURL=registry.js.map